/*
 *	Created for SALSA 0.6.0 new token semantics
 *	(July, 2003)
 *
 *	Author: Travis Desell
 *
 *	TokenInput is created by the generated code to act as 
 *	a placeholder for the return value of a message.  It then
 *	will resolve this return value by sending that value to
 *	all actors who have a placeholder for this token in a 
 *	message's arguments in their mailbox.
 */

package salsa.language;

import java.util.Vector;

public class TokenInput implements java.io.Serializable {

	/*
	 *	Used to store the return value of the token
	 */
	public Object value = null;

	/*
	 *	Used to store the actors which will need
	 *	this token to process messages.
	 */
	public Vector targetActors = new Vector();
	public Vector targetIds = new Vector();

	/*
	 *	The constructor used by token to create an
	 *	instance of a TokenInput.
	 */
	protected TokenInput() {
	}


	/*
	 *	When the token which created this TokenInput
	 *	has another TokenOutput created, that target
	 *	must be added to this TokenInput, with this
	 *	method.
	 */
	protected void addTarget(ActorReference target, String id) {
		targetActors.add(target);
		targetIds.add(id);
	}

	/*
	 *	This resolves the value returned by the
	 *	message which uses this token input by
	 *	sending that value to all its targetActors
	 *	as a "resolveToken" message.
	 */
	public void resolve(ActorReference thisActor, Object value) {
		this.value = value;

		if (targetActors.size() == 0) {
			return;
		}

		for (int i = 0; i < targetActors.size(); i++) {	
			Message message = new Message(thisActor, ((ActorReference)targetActors.get(i)), "resolveToken", null);
			Object[] arguments = { targetIds.get(i), value };
			message.setArguments( arguments );

			((ActorReference)targetActors.get(i)).send(message);
		}
	}
}
